extends Spatial

var StorageClass = preload("Storage.gd")

func _ready():
	var original = get_node("Object");
	modifyObject(original);
	
	var duplicatedNode = original.duplicate();
	add_child(duplicatedNode);
	
	if(!testNode(original, true)):
		print("There is an error inside the tests: Orginal node didn't pass it. ")
		print("Hint: Test orginal node with parameter 'silent=false' to check where is an error.")
	else:
		testNode(duplicatedNode);


###
func testNode(inNode2Test, inSilent = false):
	var rootSignalTest = testRootSignal(inNode2Test);
	var innerSignaltest = testInnerSignal(inNode2Test);
	var rootNodeGroupTest = testRootNodeGroup(inNode2Test);
	var innerNodeGroupTest = testRootNodeGroup(inNode2Test);
	var translationTestResult = translationTest(inNode2Test);
	var rootExportVarResult = testRootExportedVar(inNode2Test);
	var innerExportVarResult = testInnerExportedVar(inNode2Test)
	var variableValueRootTest = testRootVariableValue(inNode2Test);
	var variableValueInnerTest = testInternalDynamicNodeVariable(inNode2Test);
	var internalDynamicStructure = testInternalDynamicNodeStructure(inNode2Test);
	
	var summaryResult = rootSignalTest&&innerSignaltest&&rootNodeGroupTest&&innerNodeGroupTest&&translationTestResult&&rootExportVarResult&&innerExportVarResult&&variableValueRootTest&&variableValueInnerTest&&internalDynamicStructure;
	
	if(inSilent):
		return summaryResult;
	
	print(" ")
	print("----Testing signal duplication----")
	print("Root signal duplicate status: ", translateStatus2String(rootSignalTest));
	print("Inner signal duplicate status: ", translateStatus2String(innerSignaltest));
	
	
	print(" ")
	print("----Testing group duplication----")
	print("Root node group duplicate status: ", translateStatus2String(rootNodeGroupTest));
	print("Inner node group duplicate status: ", translateStatus2String(innerNodeGroupTest));
	
	print(" ")
	print("----Testing dynamic structure duplication----")
	print("Translation test: ", translateStatus2String(translationTestResult));
	print("Root export variable test: ", translateStatus2String(rootNodeGroupTest));
	print("Inner export variable test: ", translateStatus2String(innerExportVarResult));
	print("Root variable value test: ", translateStatus2String(variableValueRootTest));
	print("Inner variable val test: ", translateStatus2String(variableValueInnerTest));
	print("Inner node structure test: ", translateStatus2String(internalDynamicStructure));
	
	
	print(" ")
	print("------- Final result------- ")
	print("Summary result: ", translateStatus2String(summaryResult));
	
	return summaryResult;

###### Signals
##############
func testRootSignal(inDuplicate):
	return inDuplicate.is_connected("mouse_enter", inDuplicate, "_on_Object_mouse_enter");

func testInnerSignal(inDuplicate):
	return inDuplicate.get_node("InnerNode").is_connected("enter_screen", inDuplicate, "_on_InnerNode_enter_screen");
#func testUserSignal

###### Groups
##############
func testRootNodeGroup(inDuplicate):
	return inDuplicate.is_in_group("ENTITY");

func testInnerNodesGroup(inDuplicate):
	return inDuplicate.get_node("InnerNode").is_in_group("INNER_GROUP");
	
###### Internal Structure
################
func modifyObject(inOrginal):
	inOrginal.someStringValue = "TestValue";
	inOrginal.set_translation(Vector3(1,1,1))
	inOrginal.exportVariable = "export";
	
	var attachmentDataNode = StorageClass.new();
	attachmentDataNode.value = "InnerTestValue";
	attachmentDataNode.innerExportVar = "innerExport";
	attachmentDataNode.set_name("AdditionalDataNode");
	inOrginal.add_child(attachmentDataNode);

func testRootExportedVar(inDuplicate):
	return inDuplicate.exportVariable == "export";

func testInnerExportedVar(inDuplicate):
	if(inDuplicate.has_node("AdditionalDataNode")):
		return inDuplicate.get_node("AdditionalDataNode").innerExportVar == "innerExport";
	else:
		return false;

func testRootVariableValue(inDuplicate):
	return inDuplicate.someStringValue=="TestValue";

func testInternalDynamicNodeStructure(inDuplicate):
	return inDuplicate.has_node("AdditionalDataNode");

func testInternalDynamicNodeVariable(inDuplicate):
	if(inDuplicate.has_node("AdditionalDataNode")):
		return inDuplicate.get_node("AdditionalDataNode").value=="InnerTestValue";
	else:
		return false;

func translationTest(inDuplicate):
	return inDuplicate.get_translation() == Vector3(1,1,1);

########## Inner methods
################
func translateStatus2String(inStatus):
	if(inStatus):
		return "SUCCESS";
	else:
		return "FAIL";
